
/**********************************************************************
 * JS for New Customer Order page
 **********************************************************************/

/**********************************************************************
 * Global Variables / Config
 **********************************************************************/

/*
 * Keep global references to some Backbone objects.  These will be instantiated
 * on document load.
 */
var App, ItemDialog, OrderItems;

/*
 * Keep track of which phone number was provided by customer's database record,
 * so we can determine if the user has truly provided a different one.
 */
var customer_phone_stored = null;

/*
 * The default template syntax for underscore.js conflicts with Mako, so we
 * must override it here.
 */
_.templateSettings = {
    interpolate: /\{\{(.+?)\}\}/g
};

/*
 * Prompt user if they attempt to leave this page before creating the order.
 */
var okay_to_leave = true;       // FIXME

window.onbeforeunload = function() {
    if (! okay_to_leave) {
        return "You have unsaved changes, are you sure you want to leave this page?";
    }
}


/**********************************************************************
 * Functions
 **********************************************************************/

/*
 * Convenience function which updates the customer phone autocomplete elements,
 * whenever a customer is identified via either name or phone autocomplete.
 */
function update_customer_phone(data) {
    $('#customer_phone_known-textbox').autocomplete('option', 'disabled', true);
    $('#customer_phone_known').val(data.phone_number);
    $('#customer_phone_known-textbox').val(data.phone_number);
    $('#customer_phone_known-textbox').select();
    $('#customer_phone_known-textbox').focus();
    customer_phone_stored = data.phone_number;
    if (customer_phone_stored == '') {
        $('div.field-wrapper.update-phone').show();
        $('#update-phone').prop('checked', true);
        $('#update-phone').attr('disabled', true);
    }
    if (data.note) {
        $('#customer-notes').text(data.note);
        $('div.field-wrapper.customer-notes').show();
    }
}


/*
 * This is the callback function for the customer name autocomplete field.
 */
function customer_selected(uuid, name) {
    // FIXME url
    $.get(urls.customer_info, {'uuid': uuid}, function(data) {
        update_customer_phone(data);
    });
}


/*
 * This is the callback function for when the user clicks the "change" button
 * next to the customer name autocomplete field.
 */
function clear_customer() {
    $('#customer_phone_known').val('');
    $('#customer_phone_known-textbox').val('');
    $('#customer_phone_known-textbox').autocomplete('option', 'disabled', false);
    $('div.field-wrapper.update-phone').hide();
    $('#update-phone').prop('checked', false);
    $('#update-phone').attr('disabled', false);
    customer_phone_stored = null;
    $('div.field-wrapper.customer-notes').hide();
    $('#customer-notes').text('');
}


/*
 * This is the callback function for the customer phone autocomplete field.
 */
function customer_phone_selected(event, ui) {
    var uuid = ui.item.value;
    // FIXME url
    $.get(urls.customer_info, {'uuid': uuid}, function(data) {
        update_customer_phone(data);
        $('#customer_uuid').val(uuid);
        $('#customer_uuid-display span:first').text(data.name);
        $('#customer_uuid-textbox').hide();
        $('#customer_uuid-display').show();
    });
    return false;
}


/*
 * This function determines if a given phone number is valid, i.e. contains
 * exactly 7 or 10 digits.
 */
function validate_phone_number(text) {
    var digits = text.replace(/\D/g, '');
    return (digits.length == 7) || (digits.length == 10);
}


// /*
//  * This function prompts an employee to login, if there is no user logged in.
//  */
// function authenticate_employee() {
//     var dialog = $('#employee-login-dialog');
//     if (! dialog.length) {
//         dialog = $('<div />', {
//             id: 'employee-login-dialog'
//         });
//     }

//     function init_dialog() {
//         dialog.find('input[type="submit"]').button();
//         dialog.find('button').button();

//         dialog.find('form').submit(function() {
//             $.post($(this).attr('action'), $(this).serialize(), function(data) {
//                 if (typeof(data) == 'object') {
//                     if (data.result == 'success') {
//                         current_user = data.user_uuid;
//                         dialog.dialog('close');
//                         submit_form();
//                     } else if (data.result == 'no_user') {
//                         var msg = "Hello, " + data.employee_name + ".\n\n"
//                             + "I see you are an employee, but you do not yet have a Dtail user account.\n\n"
//                             + "You will not be able to create special orders until you have this.  Please\n"
//                             + "talk to the IT department so they can create your account.\n\n";
//                         alert(msg);
//                         dialog.load(urls.employee_login, function() {
//                             init_dialog();
//                         });
//                     } else {
//                         alert("Unexpected result:\n\n" + data.result);
//                     }
//                 } else {
//                     dialog.html(data);
//                     init_dialog();
//                 }
//             });
//             return false;
//         });

//         dialog.find('button.cancel').click(function() {
//             dialog.dialog('close');
//         });

//         with (dialog.find('#employee')) {
//             select();
//             focus();
//         }
//     }

//     dialog.load(urls.employee_login, function() {
//         init_dialog();
//         dialog.dialog({
//             title: "Employee Login",
//             width: 300,
//             height: 225
//         });
//     });
// }


/*
 * This function assembles the data hash used to submit the special order(s) to
 * the server for creation.
 */
function collect_data() {
    var data = {user: current_user};

    if ($('#customer_exists_true').is(':checked')) {
        data.customer = $('#customer_uuid').val();
        data.customer_name = null;
        data.customer_phone = $('#customer_phone_known-textbox').val();
        if ($('#update-phone').is(':checked')) {
            data.update_customer_phone = 'true';
        }
    } else {
        data.customer = null;
        data.customer_name = $('#customer_name').val();
        data.customer_phone = $('#customer_phone_unknown').val();
    }

    // We must use FormEncode-compatible keys for the product data.  It would
    // be nice if we could just send nested JSON data structures, but oh well.
    var i = 0;
    OrderItems.each(function(item) {
        data['item-' + i + '.product'] = item.get('product_uuid');
        data['item-' + i + '.product_brand'] = item.get('product_brand');
        data['item-' + i + '.product_description'] = item.get('product_description');
        data['item-' + i + '.product_size'] = item.get('product_size');
        data['item-' + i + '.quantity'] = item.get('quantity');
        data['item-' + i + '.unit_of_measure'] = item.get('unit_of_measure');
        data['item-' + i + '.discount'] = item.get('discount');
        data['item-' + i + '.note'] = item.get('note');
        ++i;
    });

    return data;
}


/*
 * This function finally submits the form data to create the new customer order.
 */
function submit_form() {
    $('body').mask("Creating Order...");
    // FIXME url
    $.post(urls.create_orders, collect_data(), function(data) {
        if (data.result == 'success') {
            okay_to_leave = true;
            // location.href = urls.orders_created; // FIXME url
            location.href = urls.create_orders; // FIXME url
        } else {
            alert("Unexpected result:\n\n" + data.result.toString());
        }
    });
}


/**********************************************************************
 * Document Load Event
 **********************************************************************/

$(function() {

    /************************************************************
     * OrderItem model
     ************************************************************/

    var OrderItem = Backbone.Model.extend({

        defaults: function() {
            return {
                product_exists: true,
                product_uuid: null,
                product_brand: '',
                // product_description: default_ambiguous_product_description,
                product_description: '', // FIXME
                product_size: '',
                product_unit_of_measure: null,
                quantity: 1,
                unit_of_measure: null,
                // discount: default_discount,
                discount: 0,    // FIXME
                note: ''
            };
        },

        validate: function(attrs, options) {
            var value;

            if (attrs.product_exists !== undefined) {

                if (attrs.product_exists) {
                    if (! attrs.product_uuid) {
                        return {
                            attr: 'product_uuid',
                            error: "Product is required."
                        };
                    }

                } else {
                    if (! attrs.product_description) {
                        return {
                            attr: 'product_description',
                            error: "Product description is required."
                        };
                    }
                }
            }

            if (attrs.quantity !== undefined) {
                value = parseInt(attrs.quantity);
                if (isNaN(value) || value != attrs.quantity) {
                    return {
                        attr: 'quantity',
                        error: "Quantity must be numeric."
                    };
                }
                attrs.quantity = value;
            }

            if (attrs.discount !== undefined) {
                if (attrs.discount) {
                    value = parseInt(attrs.discount);
                    if (isNaN(value) || value != attrs.discount || value < 0 || value > 100) {
                        return {
                            attr: 'discount',
                            error: "Discount must be a number between 0 and 100."
                        };
                    }
                }
                attrs.discount = value;
            }
        }

    });


    /*
     * We use local storage for the OrderItem collection; this avoids
     * cluttering the user's session (or worse, the database) on the server.
     */
    var OrderItemList = Backbone.Collection.extend({

        model: OrderItem,

        localStorage: new Backbone.LocalStorage('tailbone.new_customer_order')

    });

    // Create global instance of item list.
    OrderItems = new OrderItemList();


    /*
     * The editable view of an OrderItem takes the form of a dialog.
     */
    var OrderItemDialogView = Backbone.View.extend({

        el: $('#item-dialog'),

        /*
         * Cached product attributes; fetched whenever a product is selected in
         * the autocomplete field.
         */
        productAttrs: {},

        isVisible: function() {
            return this.$el.is(':visible');
        },

        setModel: function(model) {
            if (this.model) {
                this.stopListening(this.model);
            }
            if (model === undefined) {
                model = new OrderItem();
            }
            this.model = model;
            this.listenTo(this.model, 'invalid', this.showValidationError);
        },

        showValidationError: function(model) {
            var attr = model.validationError.attr;
            if (attr == 'product_uuid') {
                attr = 'product_uuid-textbox';
            }
            alert(model.validationError.error);
            with (this.$('#' + attr)) {
                select();
                focus();
            }
        },

        updateDisplay: function() {
            var that = this;

            this.$('#product_uuid-textbox').val('');
            this.$('#upc-textbox').val('');

            if (this.model.get('product_exists')) {
                this.$('#product_exists_true').click();

                if (this.model.get('product_uuid')) {
                    this.$('#product_uuid').val(this.model.get('product_uuid'));

                    // Mimic the pretty description returned by autocomplete.
                    var words = [];
                    if (this.model.get('product_brand')) {
                        words.push(this.model.get('product_brand'));
                    }
                    if (this.model.get('product_description')) {
                        words.push(this.model.get('product_description'));
                    }
                    if (this.model.get('product_size')) {
                        words.push(this.model.get('product_size'));
                    }
                    this.$('#product_uuid-display span:first').text(words.join(' '));

                    this.$('#product_uuid-textbox').hide();
                    this.$('#product_uuid-display').show();

                    this.$('#upc-input').hide();
                    var upc = this.model.get('product_upc');
                    upc = upc.slice(0, -1) + '-' + upc.slice(-1);
                    this.$('#upc-display span:first').text(upc);
                    this.$('#upc-display').show();

                } else {
                    this.$('#product_uuid').val('');
                    this.$('#product_uuid-display span:first').text('');
                    this.$('#product_uuid-display').hide();
                    this.$('#product_uuid-textbox').show();
                    this.$('#upc-display span:first').text('');
                    this.$('#upc-display').hide();
                    this.$('#upc-input').show();
                }
                
                // this.$('#product_description').val(default_ambiguous_product_description);

            } else {
                this.$('#product_exists_false').click();

                this.$('#product_uuid').val('');
                this.$('#product_uuid-display span:first').text('');
                this.$('#product_uuid-display').hide();
                this.$('#product_uuid-textbox').show();

                this.$('#upc-display span:first').text('');
                this.$('#upc-display').hide();
                this.$('#upc-input').show();

                this.$('#product_description').val(this.model.get('product_description'));
            }

            this.$('#quantity').val(this.model.get('quantity'));
            this.$('#discount').val(this.model.get('discount'));
            this.$('#note').val(this.model.get('note'));

            var updateMoreStuff = function() {
                that.updateUOMChoices();

                // Unit of measure selection must happen after we're certain
                // the available choices are accurate.
                that.$('#unit_of_measure').val(that.model.get('unit_of_measure'));

                that.updateDiscountField();
                that.updatePriceFields();
            }

            if (this.model.get('product_exists') && this.model.get('product_uuid')) {
                this.fetchProductAttrs(this.model.get('product_uuid'), updateMoreStuff);
            } else {
                updateMoreStuff();
            }
        },

        updateModel: function() {
            var attrs = {};

            if (this.$('#product_exists_true').is(':checked')) {
                attrs.product_exists = true;
                attrs.product_uuid = this.$('#product_uuid').val();
                if (attrs.product_uuid) {
                    _.extend(attrs, this.productAttrs);
                }
            } else {
                attrs.product_exists = false;
                attrs.product_uuid = null;
                attrs.product_brand = '';
                attrs.product_description = $.trim(this.$('#product_description').val());
                attrs.product_size = '';
            }

            attrs.quantity = this.$('#quantity').val();
            attrs.unit_of_measure = this.$('#unit_of_measure').val();
            attrs.discount = this.$('#discount').val();

            // Set core attributes requiring validation first.
            if (! this.model.set(attrs, {validate: true})) {
                return false;
            }

            this.model.set({
                quantity_display: attrs.quantity + " " + this.$('#unit_of_measure option:selected').text(),
                price_display: this.$('#price_after_discount').val(),
                note: this.$('#note').val()
            });

            return true;
        },

        showDialog: function(model) {
            var that = this;

            this.setModel(model);
            this.updateDisplay();

            options = {
                title: this.model.isNew() ? "Add Item" : "Edit Item",
                width: 700,
                height: 500,
                buttons: [
                    {
                        text: "OK",
                        click: function() {
                            if (that.updateModel()) {
                                if (that.model.isNew()) {
                                    OrderItems.create(that.model);
                                }
                                that.$el.dialog('close');
                            }
                        }
                    },
                    {
                        text: "Cancel",
                        click: function() {
                            that.$el.dialog('close');
                        }
                    }
                ]
            };

            this.$el.dialog(options);
            this.$('#item-tabs').tabs('option', 'active', 0);
            this.setFocus();
        },

        setFocus: function(panel) {
            if (panel == undefined) {
                panel = this.$('#item-product-tab');
                if (! panel.is(':visible')) {
                    panel = this.$('#item-quantity-tab');
                    if (! panel.is(':visible')) {
                        panel = this.$('#item-note-tab');
                    }
                }
            }

            if (panel.attr('id') == 'item-product-tab') {
                if (this.$('#product_exists_true').is(':checked')) {
                    if (! this.$('#product_uuid').val()) {
                        with (this.$('#product_uuid-textbox')) {
                            select();
                            focus();
                        }
                    }
                } else {
                    with (this.$('#product_description')) {
                        select();
                        focus();
                    }
                }

            } else if (panel.attr('id') == 'item-quantity-tab') {
                with (this.$('#quantity')) {
                    select();
                    focus();
                }
            } else { // item-note-tab
                with (this.$('#note')) {
                    select();
                    focus();
                }
            }
        },

        fetchProductAttrs: function(uuid, success) {
            var that = this;
            // FIXME url
            $.get(urls.product_info, {uuid: uuid}, function(data) {
                that.productAttrs = data;
                if (success !== undefined) {
                    success(data);
                }
            });
        },

        productSelected: function(uuid) {
            var that = this;
            this.fetchProductAttrs(uuid, function(data) {
                var upc = data.product_upc;
                upc = upc.slice(0, -1) + '-' + upc.slice(-1);
                that.$('#upc-display span:first').text(upc);
                that.$('#upc-input').hide();
                that.$('#upc-display').show();
                that.updateUOMChoices();
                that.updateDiscountField();
                that.updatePriceFields();
            });
        },

        productCleared: function() {
            this.$('#upc-textbox').val('');
            this.$('#upc-display').hide();
            this.$('#upc-input').show();
        },

        updateUOMChoices: function(product_exists, product_uuid) {
            var uom = this.$('#unit_of_measure');
            uom.empty();

            if (product_exists === undefined) {
                product_exists = this.$('#product_exists_true').is(':checked');
            }
            if (product_exists && product_uuid === undefined) {
                product_uuid = this.$('#product_uuid').val();
            }

            var uoms = null;
            if (! product_exists) {
                uoms = default_units_of_measure;
            } else if (product_uuid) {
                uoms = this.productAttrs.units_of_measure;
            }

            if (uoms) {
                _.each(uoms, function(pair) {
                    uom.append($('<option />', {
                        value: pair[0],
                        text: pair[1]
                    }));
                });
            }
        },

        updateDiscountField: function() {
            if (this.$('#product_exists_true').is(':checked') && this.$('#product_uuid').val()) {
                this.$('#discount').val(this.productAttrs.default_discount.toString());
            } else {
                this.$('#discount').val(default_discount.toString());
            }
        },

        updatePriceFields: function() {
            var product_exists = this.$('#product_exists_true').is(':checked');
            var product_uuid = this.$('#product_uuid').val();
            var unit_of_measure = this.$('#unit_of_measure').val();

            this.$('#price_per_uom').val('');
            this.$('#price_before_discount').val('');
            this.$('#price_after_discount').val('');

            var price = null;
            if (product_exists && product_uuid) {
                price = this.productAttrs.prices[unit_of_measure];
            }

            if (price) {
                this.$('#price_per_uom').val('$ ' + price);

                var quantity = this.$('#quantity').val();
                if (! this.model.validate({quantity: quantity})) {
                    var total = (price * parseInt(quantity)).toFixed(2);
                    this.$('#price_before_discount').val('$ ' + total);

                    var discount = this.$('#discount').val();
                    if (! this.model.validate({discount: discount})) {
                        total = total * (100 - parseInt(discount)) / 100;

                        // This prevents rounding, e.g. 20.196 becomes 20.19
                        // instead of 20.20 (see #1475).
                        total = Math.floor(total * 100) / 100;

                        total = total.toFixed(2);
                        this.$('#price_after_discount').val('$ ' + total);
                    }
                }
            }
        }

    });

    ItemDialog = new OrderItemDialogView();


    /*
     * The readonly view for an OrderItem is a table row, containing cells
     * with edit and delete icons.
     */
    var OrderItemView = Backbone.View.extend({

        tagName: 'tr',

        template: _.template($('#product-template').html()),

        events: {
            'click td.edit': 'editProduct',
            'click td.delete': 'promptToDeleteProduct'
        },

        initialize: function() {
            this.listenTo(this.model, 'change', this.render);
            this.listenTo(this.model, 'destroy', this.remove);
        },

        render: function() {
            this.$el.html(this.template(this.model.toJSON()));
            return this;
        },

        editProduct: function() {
            if (! ItemDialog.isVisible()) {
                ItemDialog.showDialog(this.model);
            }
        },

        promptToDeleteProduct: function() {
            if (! ItemDialog.isVisible()) {
                if (confirm("Are you sure you wish to remove this product from the order?")) {
                    this.model.destroy();
                }
            }
        }

    });


    /*
     * The main Backbone application.  This coordinates things which can't be
     * handled at the model level.
     */
    var AppView = Backbone.View.extend({

        el: $('#order-tabs'),

        initialize: function() {
            this.items_tbody = this.$('#order-items-tab #items tbody');
            // this.dialog = $('#item-dialog');
            this.listenTo(OrderItems, 'add', this.addRow);
            this.listenTo(OrderItems, 'remove', this.fixRowHighlights);
        },

        setMainFocus: function(panel) {
            if (panel == undefined) {
                panel = $('#order-customer-tab');
                if (! panel.is(':visible')) {
                    panel = $('#order-items-tab');
                }
                if (! panel.is(':visible')) {
                    panel = $('#order-note-tab');
                }
            }

            if (panel.attr('id') == 'order-customer-tab') {
                if ($('#customer_exists_true').is(':checked')) {
                    if (! $('#customer_uuid').val()) {
                        with ($('#customer_uuid-textbox')) {
                            select();
                            focus();
                        }
                    } else {
                        with ($('#customer_phone_known-textbox')) {
                            select();
                            focus();
                        }
                    }
                } else {
                    with ($('#customer_name')) {
                        select();
                        focus();
                    }
                }

            } else if (panel.attr('id') == 'order-note-tab') {
                $('#order-note-text').focus();
            }
        },

        addRow: function(product) {
            var view = new OrderItemView({model: product});
            var tr = $(view.render().el);
            tr.addClass((this.items_tbody.find('tr').length % 2) ? 'even' : 'odd');
            this.items_tbody.append(tr);
        },

        fixRowHighlights: function() {
            var odd = true;
            this.items_tbody.find('tr').each(function() {
                var wanted = odd ? 'odd' : 'even';
                var unwanted = odd ? 'even' : 'odd';
                if ($(this).hasClass(unwanted)) {
                    $(this).removeClass(unwanted);
                }
                $(this).addClass(wanted);
                odd = ! odd;
            });
        }

    });

    App = new AppView();


    /*
     * Set up the order tabs within the main screen.
     */
    $('#order-tabs').tabs({
        activate: function(event, ui) {
            App.setMainFocus(ui.newPanel);
        }
    });

    /*
     * When user edits the phone number after having selected an existing
     * customer account, show the "please update" checkbox.
     */
    $('#customer_phone_known-textbox').keypress(function(event) {
        if (customer_phone_stored !== null && event.which != 0) {
            $('div.field-wrapper.update-phone').show();
        }
    });

    /*
     * When user changes the option indicating whether the customer is already
     * in the system, toggle visibility of appropriate sections and set focus.
     */
    $('input[name="customer_exists"]').click(function() {
        if ($(this).val() == 'true') {
            $('#customer-info-unknown').hide();
            $('#customer-info-known').show();
        } else {
            $('#customer-info-known').hide();
            $('#customer-info-unknown').show();
        }
        App.setMainFocus();
    });


    /*
     * Set up the tabs within the item dialog.
     */
    $('#item-dialog #item-tabs').tabs({
        activate: function(event, ui) {
            ItemDialog.setFocus(ui.newPanel);
        }
    });
    

    /*
     * When user clicks the Add Product button, show dialog for new product.
     */
    $('#add-item').click(function() {
        if (! ItemDialog.isVisible()) {
            ItemDialog.showDialog();
        }
    });


    /*
     * When user changes the option indicating whether the product is already
     * in the system, toggle visibility of appropriate sections and set focus.
     */
    $('#item-dialog input[name="product_exists"]').click(function() {
        if ($(this).val() == 'true') {
            $('#product-info-unknown').hide();
            $('#product-info-known').show();
        } else {
            $('#product-info-known').hide();
            $('#product-info-unknown').show();
        }
        ItemDialog.updateUOMChoices();
        ItemDialog.updateDiscountField();
        ItemDialog.updatePriceFields();
        ItemDialog.setFocus();
    });


    /*
     * When user clicks Change button for product autocomplete, remove the UOM
     * choices since the new state is "product supposedly exists, but none has
     * been selected."  Also reset the UPC search.
     */
    $('#item-dialog #product_uuid-change').click(function() {
        $('#item-dialog #upc').val('');
        ItemDialog.updateUOMChoices(true, null);
        ItemDialog.updateDiscountField();
        ItemDialog.updatePriceFields();
    });


    /*
     * When user clicks UPC Fetch button, locate product via AJAX and then load
     * product data as if from autocomplete.
     */
    $('#item-dialog #upc-fetch').click(function() {
        var upc = $.trim($('#item-dialog #upc-textbox').val());
        if (! upc) {
            alert("You must provide a UPC to fetch.");
            with ($('#item-dialog #upc-textbox')) {
                select();
                focus();
            }
            return;
        }
        $('#item-dialog #product-info-known').mask("Fetching...");
        // FIXME url
        $.get(urls.product_search, {upc: upc}, function(data) {
            $('#item-dialog #product-info-known').unmask();
            if (data.product) {
                $('#item-dialog #product_uuid').val(data.product.uuid);
                $('#item-dialog #product_uuid-display span:first').text(data.product.full_description);
                $('#item-dialog #product_uuid-textbox').hide();
                $('#item-dialog #product_uuid-display').show();
                ItemDialog.productSelected(data.product.uuid);
            } else {
                alert("No product found with that UPC!");
                with ($('#item-dialog #upc-textbox')) {
                    select();
                    focus();
                }
            }
        });
    });


    /*
     * When user presses Enter while focused on the product UPC textbox,
     * simulate click of the Fetch button.
     */
    $('#item-dialog #upc-textbox').keydown(function(event) {
        if (event.which == 13) {
            $('#item-dialog #upc-fetch').click();
        }
    });


    /*
     * When user clicks Change button for product UPC, simulate clicking the
     * autocomplete Change button since that already does everything we want in
     * this situation.  But then set focus to the UPC field.
     */
    $('#item-dialog #upc-change').click(function() {
        $('#item-dialog #product_uuid-change').click();
        $('#item-dialog #upc-textbox').focus();
    });

    
    /*
     * When user changes the various fields which affect pricing, update the
     * displayed prices.
     */
    $('#item-dialog #quantity').change(function() {
        ItemDialog.updatePriceFields();
    });

    $('#item-dialog #unit_of_measure').change(function() {
        ItemDialog.updatePriceFields();
    });

    $('#item-dialog #discount').change(function() {
        ItemDialog.updatePriceFields();
    });


    /*
     * When user clicks the final submit button, validate things and then
     * prompt one last time just to be sure.
     */
    $('form[name="new_order"]').submit(function() {
        if (ItemDialog.isVisible()) {
            return false;
        }

        // if ($('#customer_exists_true').is(':checked')) {
        //     if (! $('#customer_uuid').val()) {
        //         alert("Customer is required.");
        //         $('#order-tabs').tabs('option', 'active', 0);
        //         with ($('#customer_uuid-textbox')) {
        //             select();
        //             focus();
        //         }
        //         return false;
        //     }
        //     if (! $('#customer_phone_known-textbox').val()) {
        //         alert("Customer phone number is required.");
        //         $('#order-tabs').tabs('option', 'active', 0);
        //         with ($('#customer_phone_known-textbox')) {
        //             select();
        //             focus();
        //         }
        //         return false;
        //     }
        //     if (! validate_phone_number($('#customer_phone_known-textbox').val())) {
        //         alert("Customer phone must contain exactly 7 or 10 numeric digits.");
        //         $('#order-tabs').tabs('option', 'active', 0);
        //         with ($('#customer_phone_known-textbox')) {
        //             select();
        //             focus();
        //         }
        //         return false;
        //     }
        // } else {
        //     if (! $('#customer_name').val()) {
        //         alert("Customer name is required.");
        //         with ($('#customer_name')) {
        //             select();
        //             focus();
        //         }
        //         return false;
        //     }
        //     if (! $('#customer_phone_unknown').val()) {
        //         alert("Customer phone number is required.");
        //         with ($('#customer_phone_unknown')) {
        //             select();
        //             focus();
        //         }
        //         return false;
        //     }
        // }

        // if (! OrderProducts.length) {
        //     alert("At least one product is required.");
        //     $('#order-tabs').tabs('option', 'active', 1);
        //     ItemDialog.showDialog();
        //     return false;
        // }

        if (! confirm("Are you sure you're ready to create the order?")) {
            return false;
        }

        // if (current_user) {
        //     submit_form();
        // } else {
        //     authenticate_employee();
        // }

        return false;
    });

});
